@@ -113,6 +113,16 @@ ENTRIES = [
     CE('disasm_view_unprintable_character_color', QColor, QColor(0x80, 0x40, 0x00)),
     CE('disasm_view_unknown_byte_color', QColor, QColor(0xf0, 0x00, 0x00)),
     CE('disasm_view_unknown_character_color', QColor, QColor(0xf0, 0x00, 0x00)),
+    CE('disasm_view_back_edge_color', QColor, QColor(0xf9, 0xd5, 0x77)),
+    CE('disasm_view_true_edge_color', QColor, QColor(0x79, 0xcc, 0xcd)),
+    CE('disasm_view_false_edge_color', QColor, QColor(0xf1, 0x66, 0x64)),
+    CE('disasm_view_direct_jump_edge_color', QColor, QColor(0x56, 0x5a, 0x5c)),
+    CE('disasm_view_exception_edge_color', QColor, QColor(0xf9, 0x91, 0x0a)),
+    CE('hex_view_selection_color', QColor, QColor(0xFF, 0x00, 0x00)),
+    CE('hex_view_selection_alt_color', QColor, QColor(0xA0, 0xA0, 0xA4)),
+    CE('hex_view_data_color', QColor, QColor(0x00, 0x00, 0xFF)),
+    CE('hex_view_string_color', QColor, QColor(0x00, 0xFF, 0xFF)),
+    CE('hex_view_instruction_color', QColor, QColor(0xFF, 0x00, 0xFF)),
     CE('function_table_color', QColor, QColor(0x00, 0x00, 0x00)),
     CE('function_table_syscall_color', QColor, QColor(0x00, 0x00, 0x80)),
     CE('function_table_plt_color', QColor, QColor(0x00, 0x80, 0x00)),
