@@ -3,6 +3,7 @@ from collections import defaultdict
 import logging
 from typing import Union, Optional, TYPE_CHECKING
 
+import PySide2
 from PySide2.QtWidgets import QHBoxLayout, QVBoxLayout, QApplication, QMessageBox, QMenu, QAction
 from PySide2.QtCore import Qt, Signal
 from PySide2.QtGui import QCursor
@@ -11,6 +12,7 @@ from ...logic import GlobalInfo
 from ...data.instance import ObjectContainer
 from ...utils import locate_function
 from ...data.function_graph import FunctionGraph
+from ...data.highlight_region import SynchronizedHighlightRegion
 from ...logic.disassembly import JumpHistory, InfoDock
 from ..widgets import QDisassemblyGraph, QDisasmStatusBar, QLinearDisassembly, QFeatureMap,\
     QLinearDisassemblyView, DisassemblyLevel
@@ -25,7 +27,7 @@ from ..dialogs.hook import HookDialog
 from ..dialogs.func_doc import FuncDocDialog
 from ..menus.disasm_insn_context_menu import DisasmInsnContextMenu
 from ..menus.disasm_label_context_menu import DisasmLabelContextMenu
-from .view import BaseView
+from .view import SynchronizedView
 from ..widgets import QFindAddrAnnotation, QAvoidAddrAnnotation, QBlockAnnotations
 from ..widgets.qblock_code import QVariableObj
 from ...ui.widgets.qinst_annotation import QHookAnnotation
@@ -37,10 +39,13 @@ if TYPE_CHECKING:
 _l = logging.getLogger(__name__)
 
 
-class DisassemblyView(BaseView):
+class DisassemblyView(SynchronizedView):
     """
     Disassembly View
     """
+
+    FUNCTION_SPECIFIC_VIEW = True
+
     view_visibility_changed = Signal()
     disassembly_level_changed = Signal(DisassemblyLevel)
 
@@ -121,10 +126,6 @@ class DisassemblyView(BaseView):
     # Properties
     #
 
-    @property
-    def caption(self):
-        return f'{self.base_caption}-{self.index}'
-
     @property
     def disasm(self):
         return self._flow_graph.disasm
@@ -184,6 +185,15 @@ class DisassemblyView(BaseView):
     def current_function(self) -> ObjectContainer:
         return self._current_function
 
+    @property
+    def function(self) -> ObjectContainer:
+        return self._current_function
+
+    @function.setter
+    def function(self, v):
+        if v is not self._current_function.am_obj:
+            self.display_function(v)
+
     #
     # Callbacks
     #
@@ -272,6 +282,21 @@ class DisassemblyView(BaseView):
     # UI
     #
 
+    def append_view_menu_actions(self, menu: QMenu):
+        """
+        Append a separator and general QActions for this view to a given context menu.
+        """
+        menu.addSeparator()
+        menu.addMenu(self.get_synchronize_with_submenu())
+
+    def contextMenuEvent(self, event: PySide2.QtGui.QContextMenuEvent):  # pylint: disable=unused-argument
+        """
+        Display view context menu.
+        """
+        mnu = QMenu(self)
+        mnu.addMenu(self.get_synchronize_with_submenu())
+        mnu.exec_(QCursor.pos())
+
     def instruction_context_menu(self, insn, pos):
 
         self._insn_addr_on_context_menu = insn.addr
@@ -279,7 +304,10 @@ class DisassemblyView(BaseView):
         # pass in the instruction address
         self._insn_menu.insn_addr = insn.addr
         # pop up the menu
-        self._insn_menu.qmenu(extra_entries=list(self.workspace.plugins.build_context_menu_insn(insn))).exec_(pos)
+        mnu = self._insn_menu.qmenu(extra_entries=list(self.workspace.plugins.build_context_menu_insn(insn)),
+                                    cached=False)
+        self.append_view_menu_actions(mnu)
+        mnu.exec_(pos)
 
         self._insn_addr_on_context_menu = None
 
@@ -287,7 +315,9 @@ class DisassemblyView(BaseView):
 
         self._label_addr_on_context_menu = addr
         self._label_menu.addr = addr
-        self._label_menu.qmenu().exec_(pos)
+        mnu = self._label_menu.qmenu(cached=False)
+        self.append_view_menu_actions(mnu)
+        mnu.exec_(pos)
         self._label_addr_on_context_menu = None
 
     def rename_selected_object(self):
@@ -322,6 +352,7 @@ class DisassemblyView(BaseView):
         """
         mnu = self.get_context_menu_for_selected_object()
         if mnu is not None:
+            self.append_view_menu_actions(mnu)
             mnu.exec_(QCursor.pos())
 
     def popup_jumpto_dialog(self):
@@ -506,7 +537,6 @@ class DisassemblyView(BaseView):
         self._linear_viewer.refresh()
 
     def display_function(self, function):
-
         self.jump_history.jump_to(function.addr)
         self._display_function(function)
 
@@ -676,6 +706,17 @@ class DisassemblyView(BaseView):
                     addr_to_annotations[addr].append(QAvoidAddrAnnotation(addr, qsimgrs))
         return QBlockAnnotations(addr_to_annotations, parent=qblock, disasm_view=self)
 
+    def update_highlight_regions_for_synchronized_views(self, **kwargs):  # pylint: disable=unused-argument
+        """
+        Highlight each selected instruction in synchronized views.
+        """
+        regions = []
+        for addr in self.infodock.selected_insns:
+            s = self._get_instruction_size(addr)
+            if s is not None:
+                regions.append(SynchronizedHighlightRegion(addr, s))
+        self.set_synchronized_highlight_regions(regions)
+
     #
     # Initialization
     #
@@ -716,23 +757,23 @@ class DisassemblyView(BaseView):
     def _register_events(self):
         # redraw the current graph if instruction/operand selection changes
         self.infodock.selected_insns.am_subscribe(self.redraw_current_graph)
+        self.infodock.selected_insns.am_subscribe(self.update_highlight_regions_for_synchronized_views)
         self.infodock.selected_operands.am_subscribe(self.redraw_current_graph)
         self.infodock.selected_blocks.am_subscribe(self.redraw_current_graph)
         self.infodock.hovered_block.am_subscribe(self.redraw_current_graph)
         self.infodock.hovered_edge.am_subscribe(self.redraw_current_graph)
         self.infodock.selected_labels.am_subscribe(self.redraw_current_graph)
-
-        self._feature_map.addr.am_subscribe(lambda: self._jump_to(self._feature_map.addr.am_obj))
-
-        self.workspace.current_screen.am_subscribe(self.on_screen_changed)
-
         self.infodock.qblock_code_obj_selection_changed.connect(self.redraw_current_graph)
+        self._feature_map.addr.am_subscribe(lambda: self._jump_to(self._feature_map.addr.am_obj))
+        self.workspace.current_screen.am_subscribe(self.on_screen_changed)
 
     #
     # Private methods
     #
 
     def _display_function(self, the_func):
+        self.set_synchronized_cursor_address(the_func.addr)
+
         self._current_function.am_obj = the_func
         self._current_function.am_event()
 
@@ -791,3 +832,10 @@ class DisassemblyView(BaseView):
             return next(iter(self.infodock.selected_insns))
         else:
             return None
+
+    def _get_instruction_size(self, addr: int) -> Optional[int]:
+        kb = self.workspace.instance.project.kb
+        f = kb.functions.floor_func(addr)
+        if f is None:
+            return None
+        return f.instruction_size(addr)
