@@ -1,23 +1,31 @@
 import os
 import logging
+from typing import Optional
 
 from PySide2.QtWidgets import QFrame, QHBoxLayout, QLabel, QPushButton, QFileDialog, QComboBox
-from PySide2.QtCore import Qt
 
 from ..menus.disasm_options_menu import DisasmOptionsMenu
+from ..toolbars import NavToolbar
+from .qdisasm_base_control import DisassemblyLevel
 from .qdisasm_graph import QDisassemblyGraph
 from .qlinear_viewer import QLinearDisassembly
 
+
 _l = logging.getLogger(__name__)
 
 
 class QDisasmStatusBar(QFrame):
+    """
+    Status and control bar for disassembly views
+    """
+
     def __init__(self, disasm_view, parent=None):
-        super(QDisasmStatusBar, self).__init__(parent)
+        super().__init__(parent)
 
         self.disasm_view = disasm_view
 
         # widgets
+        self._nav_toolbar: Optional[NavToolbar] = None
         self._function_label: QLabel = None
         self._options_menu: DisasmOptionsMenu = None
         self._view_combo: QComboBox = None
@@ -49,6 +57,12 @@ class QDisasmStatusBar(QFrame):
     #
 
     def _init_widgets(self):
+        self._nav_toolbar = NavToolbar(
+            self.disasm_view.jump_history,
+            self.disasm_view.jump_back,
+            self.disasm_view.jump_forward,
+            self.disasm_view.jump_to_history_position,
+            True, self)
 
         # current function
         self._function_label = QLabel()
@@ -58,6 +72,15 @@ class QDisasmStatusBar(QFrame):
         self._view_combo.addItem("Graph Disassembly", QDisassemblyGraph)
         self._view_combo.activated.connect(self._view_combo_changed)
         self.disasm_view.view_visibility_changed.connect(self._update_view_combo)
+        self._update_view_combo()
+
+        self._disasm_level_combo = QComboBox(self)
+        self._disasm_level_combo.addItem("Machine Code", DisassemblyLevel.MachineCode)
+        self._disasm_level_combo.addItem("Lifter IR", DisassemblyLevel.LifterIR)
+        self._disasm_level_combo.addItem("AIL", DisassemblyLevel.AIL)
+        self._disasm_level_combo.activated.connect(self._disasm_level_combo_changed)
+        self.disasm_view.disassembly_level_changed.connect(self._update_disasm_level_combo)
+        self._update_disasm_level_combo()
 
         # options button
         option_btn = QPushButton()
@@ -71,11 +94,15 @@ class QDisasmStatusBar(QFrame):
 
         layout = QHBoxLayout()
         layout.setContentsMargins(2, 2, 2, 2)
+
+        layout.addWidget(self._nav_toolbar.qtoolbar())
         layout.addWidget(self._function_label)
 
         layout.addStretch(0)
+
         layout.addWidget(saveimage_btn)
         layout.addWidget(self._view_combo)
+        layout.addWidget(self._disasm_level_combo)
         layout.addWidget(option_btn)
         layout.setContentsMargins(0, 0, 0, 0)
 
@@ -99,6 +126,15 @@ class QDisasmStatusBar(QFrame):
         index = self._view_combo.findData(graph_type)
         self._view_combo.setCurrentIndex(index)
 
+    def _disasm_level_combo_changed(self, index:int):
+        new_level = self._disasm_level_combo.itemData(index)
+        self.disasm_view.set_disassembly_level(new_level)
+
+    def _update_disasm_level_combo(self):
+        new_level = self.disasm_view.disassembly_level
+        index = self._disasm_level_combo.findData(new_level)
+        self._disasm_level_combo.setCurrentIndex(index)
+
     def _update_function_address(self):
         if self.function_address is not None:
             self._function_label.setText("Function %x" % self.function_address)
