@@ -1,20 +1,15 @@
 from typing import TYPE_CHECKING, Callable
 import logging
 import traceback
-from string import ascii_uppercase
 
-from PySide2.QtCore import Qt, QSettings
-
-from angr.knowledge_plugins.functions.function import Function as angrFunc
-from angr.knowledge_plugins import Function
+from angr.knowledge_plugins.functions.function import Function
 from angr import StateHierarchy
 
 from ..config import Conf
 from ..data.instance import ObjectContainer
 from ..data.jobs import CodeTaggingJob, PrototypeFindingJob, VariableRecoveryJob, FlirtSignatureRecognitionJob
 from .views import (FunctionsView, DisassemblyView, SymexecView, StatesView, StringsView, ConsoleView, CodeView,
-                    InteractionView, PatchesView, DependencyView, ProximityView, TypesView, HexView)
-from .widgets.qsmart_dockwidget import QSmartDockWidget
+                    InteractionView, PatchesView, DependencyView, ProximityView, TypesView, HexView, LogView)
 from .view_manager import ViewManager
 from .menus.disasm_insn_context_menu import DisasmInsnContextMenu
 
@@ -36,9 +31,6 @@ class Workspace:
 
         self.main_window: 'MainWindow' = main_window
         self._instance = instance
-        self.is_split = False
-        self.split_tab_id = 0
-        self.last_unsplit_view = None
         instance.workspace = self
 
         self.view_manager: ViewManager = ViewManager(self)
@@ -62,23 +54,17 @@ class Workspace:
                 DependencyView(self, 'center')
             )
         self.default_tabs += [
-            SymexecView(self, 'center'),
-            StatesView(self, 'center'),
             StringsView(self, 'center'),
             PatchesView(self, 'center'),
+            SymexecView(self, 'center'),
+            StatesView(self, 'center'),
             InteractionView(self, 'center'),
             ConsoleView(self, 'bottom'),
+            LogView(self, 'bottom'),
         ]
 
-        #
-        # Save initial splitter state
-        #
-
-        self.splitter_state = QSettings()
-        self.splitter_state.setValue("splitterSizes", self._main_window.central_widget_main.saveState())
-
         for tab in self.default_tabs:
-            self.add_view(tab, tab.caption, tab.category)
+            self.add_view(tab)
 
     #
     # Properties
@@ -96,13 +82,26 @@ class Workspace:
     # Events
     #
 
-    def on_function_selected(self, func):
+    def on_function_selected(self, func: Function):
+        """
+        Callback function triggered when a new function is selected in the function view.
 
-        self._get_or_create_disassembly_view().display_function(func)
-        codeview = self.view_manager.first_view_in_category('pseudocode')
-        if codeview is not None and codeview.is_shown():
-            codeview.function.am_obj = func
-            codeview.function.am_event(focus=True)
+        :param func:    The function that is selected.
+        :return:        None
+        """
+
+        # Ask all current views to display this function
+
+        current_view = self.view_manager.current_tab
+        if current_view is None or not current_view.FUNCTION_SPECIFIC_VIEW:
+            # we don't have a current view or the current view does not have function-specific content. create a
+            # disassembly view to display the selected function.
+            disasm_view = self._get_or_create_disassembly_view()
+            disasm_view.display_function(func)
+            self.view_manager.raise_view(disasm_view)
+        else:
+            # ask the current view to display this function
+            current_view.function = func
 
     def on_cfg_generated(self):
 
@@ -177,90 +176,24 @@ class Workspace:
     # Public methods
     #
 
-    def new_disassembly_view(self):
+    def new_disassembly_view(self) -> DisassemblyView:
         """
-        Add a new disassembly view into the
-        central_widget with a unique id
-
-        :return:    None
+        Add a new disassembly view.
         """
-
-        dis_views = self.view_manager.views_by_category['disassembly']
-        dis_ids = [view.index for view in dis_views]
-        dis_ids.sort()
-        missing_ids = sorted(set(range(dis_ids[0], dis_ids[-1])) - set(dis_ids))
         new_view = DisassemblyView(self, 'center')
-        if missing_ids:
-            new_view.index = missing_ids[0]
-        else:
-            new_view.index = dis_ids[-1]+1
-        self.add_view(new_view, new_view.caption, new_view.category)
+        self.add_view(new_view)
         self.raise_view(new_view)
         if self.instance.binary_path is not None:
             self.on_cfg_generated()
         # TODO move new_view tab to front of dock
+        return new_view
 
-    def split_view(self):
-        """
-        Split the view into two panes and shift
-        the current tab into the second pane
-
-        :return:    None
-        """
-
-        window_id = self.view_manager.get_current_tab_id() + 1
-        if self.is_split is False:
-            self._main_window.central_widget.removeDockWidget(self.view_manager.docks[window_id])
-            dock_area = ViewManager.DOCKING_POSITIONS.get(self.default_tabs[window_id].default_docking_position,
-                                                          Qt.RightDockWidgetArea)
-            dock = QSmartDockWidget(self.default_tabs[window_id].caption, parent=self.default_tabs[window_id])
-            self._main_window.central_widget2.show()
-            self._main_window.central_widget2.addDockWidget(dock_area, dock)
-            dock.setWidget(self.default_tabs[window_id])
-            self.is_split = True
-            self.split_tab_id = window_id
-            self.last_unsplit_view = dock
-            self._main_window.central_widget_main.setStretchFactor(1,1)
-
-    def add_view(self, view, caption, category):
-        self.view_manager.add_view(view, caption, category)
+    def add_view(self, view):
+        self.view_manager.add_view(view)
 
     def remove_view(self, view):
         self.view_manager.remove_view(view)
 
-    def unsplit_view(self):
-        """
-        Unsplit view if it is already split
-
-        :return:    None
-        """
-
-        if self.is_split is True:
-            window_id = self.split_tab_id
-            self._main_window.central_widget2.hide()
-            self._main_window.central_widget2.removeDockWidget(self.last_unsplit_view)
-            dock_area = ViewManager.DOCKING_POSITIONS.get(self.default_tabs[window_id].default_docking_position,
-                                                          Qt.RightDockWidgetArea)
-            dock = QSmartDockWidget(self.default_tabs[window_id].caption, parent=self.default_tabs[window_id])
-            self._main_window.central_widget.addDockWidget(dock_area, dock)
-            dock.setWidget(self.default_tabs[window_id])
-            self.view_manager.docks[window_id] = dock
-            self._main_window.central_widget_main.setStretchFactor(1,0)
-            self._main_window.central_widget_main.restoreState(self.splitter_state.value("splitterSizes"))
-            self.view_manager.tabify_center_views()
-            self.is_split = False
-
-    def toggle_split(self):
-        """
-        Toggles the split state
-        :return:    None
-        """
-
-        if self.is_split is False:
-            self.split_view()
-        else:
-            self.unsplit_view()
-
     def raise_view(self, view):
         """
         Find the dock widget of a view, and then bring that dockable to front.
@@ -370,7 +303,6 @@ class Workspace:
         view.function.am_obj = func
         view.function.am_event(focus=True, focus_addr=curr_ins)
 
-
     def create_simulation_manager(self, state, state_name, view=None):
 
         inst = self.instance
@@ -417,6 +349,40 @@ class Workspace:
         self.raise_view(view)
         view.setFocus()
 
+    def create_and_show_linear_disassembly_view(self):
+        """
+        Create a new disassembly view and select the Linear disassembly mode.
+        """
+        view = self.new_disassembly_view()
+        view.display_linear_viewer()
+        self.raise_view(view)
+        view.setFocus()
+
+    def create_and_show_graph_disassembly_view(self):
+        """
+        Create a new disassembly view and select the Graph disassembly mode.
+        """
+        view = self.new_disassembly_view()
+        view.display_disasm_graph()
+        self.raise_view(view)
+        view.setFocus()
+
+    def create_and_show_hex_view(self):
+        """
+        Create and show a new hex view.
+        """
+        view = self._create_hex_view()
+        self.raise_view(view)
+        view.setFocus()
+
+    def show_pseudocode_view(self):
+        """
+        Create code view if it does not exist, then show code view.
+        """
+        view = self._get_or_create_pseudocode_view()
+        self.raise_view(view)
+        view.setFocus()
+
     def show_hex_view(self):
         view = self._get_or_create_hex_view()
         self.raise_view(view)
@@ -462,31 +428,39 @@ class Workspace:
         self.raise_view(view)
         view.setFocus()
 
+    def show_log_view(self):
+        view = self._get_or_create_log_view()
+        self.raise_view(view)
+        view.setFocus()
+
     #
     # Private methods
     #
 
     def _get_or_create_disassembly_view(self) -> DisassemblyView:
-        # Take the first disassembly view
-        if len(self.view_manager.views_by_category['disassembly']) == 1:
+        view = self.view_manager.current_view_in_category('disassembly')
+        if view is None:
             view = self.view_manager.first_view_in_category('disassembly')
-        else:
-            view = self.view_manager.current_view_in_category('disassembly')
-
         if view is None:
-            # Create a new disassembly view
             view = DisassemblyView(self, 'center')
-            self.add_view(view, view.caption, view.category)
+            self.add_view(view)
             view.reload()
 
         return view
 
-    def _get_or_create_hex_view(self):
+    def _create_hex_view(self) -> HexView:
+        """
+        Create a new hex view.
+        """
+        view = HexView(self, 'center')
+        self.add_view(view)
+        return view
+
+    def _get_or_create_hex_view(self) -> HexView:
         view = self.view_manager.first_view_in_category('hex')
 
         if view is None:
-            view = HexView(self, 'center')
-            self.add_view(view, view.caption, view.category)
+            view = self._create_hex_view()
 
         return view
 
@@ -497,7 +471,7 @@ class Workspace:
         if view is None:
             # Create a new pseudo-code view
             view = CodeView(self, 'center')
-            self.add_view(view, view.caption, view.category)
+            self.add_view(view)
 
         return view
 
@@ -508,7 +482,7 @@ class Workspace:
         if view is None:
             # Create a new symexec view
             view = SymexecView(self, 'center')
-            self.add_view(view, view.caption, view.category)
+            self.add_view(view)
 
         return view
 
@@ -519,7 +493,7 @@ class Workspace:
         if view is None:
             # Create a new states view
             view = StatesView(self, 'center')
-            self.add_view(view, view.caption, view.category)
+            self.add_view(view)
 
         return view
 
@@ -530,7 +504,7 @@ class Workspace:
         if view is None:
             # Create a new states view
             view = StringsView(self, 'center')
-            self.add_view(view, view.caption, view.category)
+            self.add_view(view)
 
         return view
 
@@ -541,7 +515,7 @@ class Workspace:
         if view is None:
             # Create a new states view
             view = PatchesView(self, 'center')
-            self.add_view(view, view.caption, view.category)
+            self.add_view(view)
 
         return view
 
@@ -550,7 +524,7 @@ class Workspace:
         if view is None:
             # Create a new interaction view
             view = InteractionView(self, 'center')
-            self.add_view(view, view.caption, view.category)
+            self.add_view(view)
         return view
 
     def _get_or_create_types_view(self):
@@ -558,7 +532,7 @@ class Workspace:
         if view is None:
             # Create a new interaction view
             view = TypesView(self, 'center')
-            self.add_view(view, view.caption, view.category)
+            self.add_view(view)
         return view
 
     def _get_or_create_proximity_view(self) -> ProximityView:
@@ -568,7 +542,7 @@ class Workspace:
         if view is None:
             # Create a new proximity view
             view = ProximityView(self, 'center')
-            self.add_view(view, view.caption, view.category)
+            self.add_view(view)
 
         return view
 
@@ -579,10 +553,22 @@ class Workspace:
         if view is None:
             # Create a new console view
             view = ConsoleView(self, 'bottom')
-            self.add_view(view, view.caption, view.category)
+            self.add_view(view)
 
         return view
 
+    def _get_or_create_log_view(self) -> LogView:
+        # Take the first log view
+        view = self.view_manager.first_view_in_category("log")
+
+        if view is None:
+            # Create a new log view
+            view = LogView(self, 'bottom')
+            self.add_view(view)
+
+        return view
+
+
     def _get_or_create_functions_view(self) -> FunctionsView:
         # Take the first functions view
         view = self.view_manager.first_view_in_category("functions")
@@ -590,7 +576,7 @@ class Workspace:
         if view is None:
             # Create a new functions view
             view = FunctionsView(self, 'left')
-            self.add_view(view, view.caption, view.category)
+            self.add_view(view)
 
         return view
 
@@ -600,7 +586,7 @@ class Workspace:
 
     # TODO: should these be removed? Nobody is using them and there is equivalent functionality elsewhere.
 
-    def set_cb_function_backcolor(self, callback: Callable[[angrFunc], None]):
+    def set_cb_function_backcolor(self, callback: Callable[[Function], None]):
         fv = self.view_manager.first_view_in_category('functions')  # type: FunctionsView
         if fv:
             fv.backcolor_callback = callback
